'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var errors = require('@backstage/errors');
var pluginScaffolderBackend = require('@backstage/plugin-scaffolder-backend');
var fetch = require('node-fetch');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fetch__default = /*#__PURE__*/_interopDefaultLegacy(fetch);

const createAzurePipelineAction = (options) => {
  const { integrations } = options;
  return pluginScaffolderBackend.createTemplateAction({
    id: "azure:pipeline:create",
    schema: {
      input: {
        required: [
          "organization",
          "project",
          "folder",
          "name",
          "repositoryId",
          "repositoryName"
        ],
        type: "object",
        properties: {
          organization: {
            type: "string",
            title: "Organization",
            description: "The name of the Azure DevOps organization."
          },
          project: {
            type: "string",
            title: "Project",
            description: "The name of the Azure project."
          },
          folder: {
            type: "string",
            title: "Folder",
            description: "The name of the folder of the pipeline."
          },
          name: {
            type: "string",
            title: "Name",
            description: "The name of the pipeline."
          },
          repositoryId: {
            type: "string",
            title: "Repository ID",
            description: "The ID of the repository."
          },
          repositoryName: {
            type: "string",
            title: "Repository Name",
            description: "The name of the repository."
          },
          yamlPath: {
            type: "string",
            title: "Azure DevOps Pipelines Definition",
            description: "The location of the Azure DevOps Pipeline definition file. Defaults to /azure-pipelines.yaml"
          },
          token: {
            title: "Authenticatino Token",
            type: "string",
            description: "The token to use for authorization."
          }
        }
      }
    },
    async handler(ctx) {
      var _a;
      const {
        organization,
        project,
        folder,
        name,
        repositoryId,
        yamlPath,
        repositoryName
      } = ctx.input;
      const host = "dev.azure.com";
      const integrationConfig = integrations.azure.byHost(host);
      if (!integrationConfig) {
        throw new errors.InputError(
          `No matching integration configuration for host ${host}, please check your integrations config`
        );
      }
      if (!integrationConfig.config.token && !ctx.input.token) {
        throw new errors.InputError(`No token provided for Azure Integration ${host}`);
      }
      const token = (_a = ctx.input.token) != null ? _a : integrationConfig.config.token;
      ctx.logger.info(
        `Creating an Azure pipeline for the repository ${repositoryName} with the ID ${repositoryId}.`
      );
      await fetch__default["default"](
        `https://dev.azure.com/${organization}/${project}/_apis/pipelines?api-version=6.1-preview.1`,
        {
          method: "POST",
          headers: {
            "Content-Type": "application/json",
            Accept: "application/json",
            Authorization: `Basic ${Buffer.from(`PAT:${token}`).toString(
              "base64"
            )}`,
            "X-TFS-FedAuthRedirect": "Suppress"
          },
          body: JSON.stringify({
            folder,
            name,
            configuration: {
              type: "yaml",
              path: yamlPath || "/azure-pipelines.yaml",
              repository: {
                id: repositoryId,
                name: repositoryName,
                type: "azureReposGit"
              }
            }
          })
        }
      ).then((response) => {
        if (response.ok) {
          ctx.logger.info(
            `Successfully created ${name} Azure pipeline in ${folder}.`
          );
        } else {
          ctx.logger.error(
            `Failed to create Azure pipeline. Status code ${response.status}.`
          );
        }
        return response.json();
      }).then((data) => {
        ctx.logger.info(`The Azure pipeline ID is ${data.id}.`);
        ctx.output("pipelineId", data.id.toString());
        ctx.output("pipelineUrl", data._links.web.href);
      });
    }
  });
};

const permitAzurePipelineAction = (options) => {
  const { integrations } = options;
  return pluginScaffolderBackend.createTemplateAction({
    id: "azure:pipeline:permit",
    schema: {
      input: {
        required: [
          "organization",
          "project",
          "resourceId",
          "resourceType",
          "authorized",
          "pipelineId"
        ],
        type: "object",
        properties: {
          organization: {
            type: "string",
            title: "Organization",
            description: "The name of the Azure DevOps organization."
          },
          project: {
            type: "string",
            title: "Project",
            description: "The name of the Azure project."
          },
          resourceId: {
            type: "string",
            title: "Resource ID",
            description: "The resource ID."
          },
          resourceType: {
            type: "string",
            title: "Resource Type",
            description: "The type of the resource (e.g. endpoint)."
          },
          authorized: {
            type: "boolean",
            title: "Authorized",
            description: "A true or false authorization indicator."
          },
          pipelineId: {
            type: "string",
            title: "Pipeline ID",
            description: "The pipeline ID."
          },
          token: {
            title: "Authenticatino Token",
            type: "string",
            description: "The token to use for authorization."
          }
        }
      }
    },
    async handler(ctx) {
      var _a;
      const {
        organization,
        project,
        resourceId,
        resourceType,
        authorized,
        pipelineId
      } = ctx.input;
      const host = "dev.azure.com";
      const integrationConfig = integrations.azure.byHost(host);
      if (!integrationConfig) {
        throw new errors.InputError(
          `No matching integration configuration for host ${host}, please check your integrations config`
        );
      }
      if (!integrationConfig.config.token && !ctx.input.token) {
        throw new errors.InputError(`No token provided for Azure Integration ${host}`);
      }
      const token = (_a = ctx.input.token) != null ? _a : integrationConfig.config.token;
      if (ctx.input.authorized === true) {
        ctx.logger.info(
          `Authorizing Azure pipeline with ID ${pipelineId} for ${resourceType} with ID ${resourceId}.`
        );
      } else {
        ctx.logger.info(
          `Unauthorizing Azure pipeline with ID ${pipelineId} for ${resourceType} with ID ${resourceId}.`
        );
      }
      await fetch__default["default"](
        `https://dev.azure.com/${organization}/${project}/_apis/pipelines/pipelinepermissions/${resourceType}/${resourceId}?api-version=7.1-preview.1`,
        {
          method: "PATCH",
          headers: {
            "Content-Type": "application/json",
            Accept: "application/json",
            Authorization: `Basic ${Buffer.from(`PAT:${token}`).toString(
              "base64"
            )}`,
            "X-TFS-FedAuthRedirect": "Suppress"
          },
          body: JSON.stringify({
            pipelines: [
              {
                authorized,
                id: parseInt(pipelineId, 10)
              }
            ]
          })
        }
      ).then((response) => {
        if (response.ok) {
          ctx.logger.info(
            `Successfully changed the Azure pipeline permissions.`
          );
        } else {
          ctx.logger.error(
            `Failed to change the Azure pipeline permissions. Status code ${response.status}.`
          );
        }
      });
    }
  });
};

const runAzurePipelineAction = (options) => {
  const { integrations } = options;
  return pluginScaffolderBackend.createTemplateAction({
    id: "azure:pipeline:run",
    schema: {
      input: {
        required: ["organization", "pipelineId", "project"],
        type: "object",
        properties: {
          organization: {
            type: "string",
            title: "Organization",
            description: "The name of the Azure DevOps organization."
          },
          pipelineId: {
            type: "string",
            title: "Pipeline ID",
            description: "The pipeline ID."
          },
          project: {
            type: "string",
            title: "Project",
            description: "The name of the Azure project."
          },
          branch: {
            title: "Repository Branch",
            type: "string",
            description: "The branch of the pipeline's repository."
          },
          token: {
            title: "Authentication Token",
            type: "string",
            description: "The token to use for authorization."
          }
        }
      }
    },
    async handler(ctx) {
      var _a;
      const { organization, pipelineId, project, branch } = ctx.input;
      const host = "dev.azure.com";
      const integrationConfig = integrations.azure.byHost(host);
      if (!integrationConfig) {
        throw new errors.InputError(
          `No matching integration configuration for host ${host}, please check your integrations config`
        );
      }
      if (!integrationConfig.config.token && !ctx.input.token) {
        throw new errors.InputError(`No token provided for Azure Integration ${host}`);
      }
      const token = (_a = ctx.input.token) != null ? _a : integrationConfig.config.token;
      ctx.logger.info(`Running Azure pipeline with the ID ${pipelineId}.`);
      await fetch__default["default"](
        `https://dev.azure.com/${organization}/${project}/_apis/pipelines/${pipelineId}/runs?api-version=6.1-preview.1`,
        {
          method: "POST",
          headers: {
            "Content-Type": "application/json",
            Accept: "application/json",
            Authorization: `Basic ${Buffer.from(`PAT:${token}`).toString(
              "base64"
            )}`,
            "X-TFS-FedAuthRedirect": "Suppress"
          },
          body: JSON.stringify({
            resources: {
              repositories: {
                self: {
                  refName: `refs/heads/${branch != null ? branch : "main"}`
                }
              }
            }
          })
        }
      ).then((response) => {
        if (response.ok) {
          ctx.logger.info(`Successfully ran Azure pipeline.`);
        } else {
          ctx.logger.error(
            `Failed to run Azure pipeline. Status code ${response.status}.`
          );
        }
      });
    }
  });
};

exports.createAzurePipelineAction = createAzurePipelineAction;
exports.permitAzurePipelineAction = permitAzurePipelineAction;
exports.runAzurePipelineAction = runAzurePipelineAction;
//# sourceMappingURL=index.cjs.js.map
